% Description: This script shows a simple implementation of a sonud coding
% strategy and synthesis through a VoCoder. This code was used as
% educational tool during the Medical Bionics Summer School in Croatia.
% 
% Author: Waldo Nogueira
% Copyright: Auditory Prosthetic Group, German Hearing Center, Medical University Hannover
% Date: 01/09/2016
clear;
close all;
% Capture the sound
[a,fsIn] = wavread('asa.wav');
fs   = 16000;
a = resample(a,fs,fsIn);

% Play the sound
soundsc(a,fs);

% Compute the FFT
b = fft(a);

% Compute the magnitude of the FFT
c = abs(b);

% Plot the magnitude spectrum in dB
figure;
plot(20*log10(c));
grid on;

% Now compute the sort time fourier transform
stimRate = 1000;
hop = floor(fs/stimRate);
nFFT = 256;
%hop  = 64;
X =  stft(a(:)', nFFT, nFFT, hop);
% Plot the Spectrogram
figure;imagesc(20*log10(abs(X)));
%X = X';
% Combine FFT bands into channels
nChans = 16;
iStartArray = [2     6     8     10    12    14    16    18    20    25    29    35    41    49    58    69] + 1;
iEndArray   = [5     7     9    11    13    15    17    19    24    28    34    40    48    57    68    127] + 1;

 for iChan = 1:nChans
        iMid = floor((iStartArray(iChan)+iEndArray(iChan))/2);

        if(iStartArray(iChan) == iEndArray(iChan))
            Yenv(iChan,:) = sqrt(abs(X(iStartArray(iChan),:)).^2);
        else
            Yenv(iChan,:) = sqrt(sum(abs(X(iStartArray(iChan):iEndArray(iChan),:)).^2));
        end
 end
 
 % Plot Envelopes
 figure;imagesc(Yenv);
 
 % Mapping-Compression
 M = [1000 1000 1000 1000 1000 1000 1000 1000 1000 1000 1000 1000 1000 1000 1000 1000 1000];
 T = [50 50 50 50 50 50 50 50 50 50 50 50 50 50 50 50 50];
 IDR = 60;
 
 
 for iChan=1:nChans
     Ymap(iChan,:) = (M(iChan)-T(iChan))/IDR*log10(Yenv(iChan,:))+T(iChan);
 end
 
 for iChan = 1:nChans
    Yres(iChan,:) = resample(Yenv(iChan,:),fs,floor(fs/hop));
 end
 % Synthesize with sinouids
 freqs = [100 300 500 700 1000 1300 2000 2300 2500 3000 3500 4000 4500 5000 6000 7000];
 

 for iChan=1:nChans
    Ysin(iChan,:) = sin(2*pi*freqs(iChan)*[1:length(a)]/(fs));
 end
 
 Yout = sum(Yres.*Ysin(:,[1:length(Yres)]));
 
 soundsc(Yout,fs);

